=head1 NAME

Model - Abstract Class for Simple Statistical Models

=head1 SYNOPSIS

    use Model;

    # First create an Model object
    my $brain = Model->load("data/Brain.txt");

    # Pick an age for a fetus (the predictor variable):
    my $predictor = 15;

    # Pick a measured weight of the brain:
    my $observed = 100;

    # Get the units of the predictor variable:
    my $p_units = $brain->p_units()

    # Get the units of the response variable:
    my $r_units = $brain->r_units();

    # Get the expected value (response variable) at the given age:
    $response = $brain->predict($predictor);

    # Get the standard deviation at the given age:
    $sd = $brain->sd($predictor);

    # Get the percentile of the observation at the given age:
    $percentile = $brain->sd($percentile);

    # Print some information about the observation
    print "The $predictor $p_units old fetus has a brain weight of " .
          "$observed.  This weight is in the $percentile percentile, " .
	  "and the expected weight was $response with a standard deviation " .
	  "of $sd.\n";

=head1 DESCRIPTION

Model is an abstract class from which other statistical models can be derived.
(see Model::Polynomial and Model::Table for specific instances).  All of its
functions except load() will simply die with a "function not implemented"
error.  Thus, these function absolutely must be overridden in a derived class.

=head1 APPENDIX

The Model class implements the following functions

=cut

package Model;

use warnings;
use strict;

=head2 load()

    Model->load($filename)

load() reads the file $filename and finds the first non-commented line which
must look like

  [ModelName]

load() then attempts to load the module Model/ModelName.pm, and returns the
value of

  Model::ModelName->load($filename)

Thus, the load function can be used to automatically select, load, and return
a reference to a model.

=cut

sub load {
    my($cn, $fn) = @_;

    my $fh;
    open($fh, "<$fn") || die "Unable to open $fn: $!\n";

    my $model_name;
    foreach my $line (<$fh>) {
	chomp($line);

	$line =~ s/#.*//;

	if($line =~ m/^\s*$/) {
	    next;
	}

	if($line =~ m/^\s*\[(\S+)\]\s*$/) {
	    $model_name = $1;
	    last;
	} else {
	    die "$fn: unable to parse model line\n";
	}
    }
    close($fh);

    require "Model/$model_name.pm";
    return "Model::$model_name"->load($fn);
}

=head2 predict()

    predict($predictor)

predict() will return the expected response for the predictor $predictor.

=cut

sub predict {
    die "predict() not implemented\n";
}


=head2 sd()

    sd($predictor)

sd() will return the standard deviation of the distribution of the response
variable for the predictor $predictor.

=cut

sub sd {
    die "sd() not implemented\n";
}


=head2 percentile()

    percentile($predictor, $x)

percentile() will return the percentile of the response observation $x for the
predictor $predictor.

=cut

sub percentile {
    die "percentile() not implemented\n";
}

=head2 p_units

    p_units()

p_units() returns the name of the units of the predictor variable.

=cut

sub p_units {
    die "x_units() not implemented\n";
}

=head2 r_unit()

    r_units()

r_units returns the name of the units of the response variable.

=cut

sub r_units {
    die "y_units() not implemented\n";
}

1;
