=head1 NAME

Model::Polynomial - Class for Polynomial Based Statistical Models

=head1 DESCRIPTION

Model::Polynomial is an implementation of the Model class.  The mean value is
computed by evaluation of a polynomial, and the standard deviation is computed
by evalution of another polynomial.  Percentiles are computed assuming a normal
distribution.

=head1 APPENDIX

When load is called on a file, in addition to a "[Polynomial]" line, the file
is expected to have several name/value pairs:

=head2 p_units

    p_units = STRING

p_units is a string containing the name of the predictor variable.

=head2 r_units

    r_units = STRING

r_units is a string containing the name of the response variable.

=head2 u_coef

    u_coef = NUM_RANGE

u_coef is a list of numbers (separated by spaces) corresponding to coefficients
of the polynomial describing the mean.  For example u_coef = 1 2 3 would
correspond to the polynomial x^2 + 2x + 3.

=head2 u_range

    u_range = RANGE

u_range is a list of two numbers indicating the minimum and maximum values to
which u_coef should be applied.

=head2 sd_coef

    sd_coef = NUM_RANGE

sd_coef is a list of numbers (separated by spaces) corresponding to coefficients
of the polynomial describing the standard deviation.

=head2 sd_range

    sd_range = RANGE

sd_range is a list of two numbers indicating the minimum and maximum values to
which sd_coef should be applied.

=head1 EXAMPLE

The following file evaluates a number of weeks between 11 and 42 and predicts a
measurement in milimeters.  The polynomial describing the mean is
2.19221x - 12.2185; the polynomial describing the standard deviation is
0.176912x + 1.24934.

  [Polynomial]
  p_units  = weeks
  r_units  = mm
  u_coef   = 2.19221 -12.2185
  u_range  = 11 42
  sd_coef  = 0.176912 1.24934
  sd_range = 11 42

=cut

package Model::Polynomial;
@ISA = ("Model");

use Statistics::Distributions;

sub load {
    my($cn, $fn) = @_;
    my $this = bless {};

    my($fh);
    open($fh, "<$fn") || die "Unable to open $fn: $!\n";

    # Get the variables
    my $ispolynomial = 0;
    my $ln = 0;
    foreach my $line (<$fh>) {
	chomp($line);
	++$ln;

	$line =~ s/#.*//;

	if($line =~ m/^\s*$/) {
	    next;
	}

	if(! $ispolynomial) {
	    if($line =~ m/^\s*\[Polynomial\]\s*$/) {
		$ispolynomial = 1;
		next;
	    } else {
		last;
	    }
	}

	my $listref;
	if($line =~ m/^\s*(\S+)\s*=\s*(.*)$/) {
	    if($1 eq "u_coef") {
		$listref = [reverse split /\s+/, $2];
		if(@$listref < 1) {
		    die "$fn:$ln: u_coef must specify at least one number\n";
		}
		foreach(@$listref) {
		    if(! _is_num($_)) {
			die "$fn:$ln: u_coef must be a list of numbers\n";
		    }
		}
		$this->{"u_coef"} = $listref;
	    } elsif($1 eq "sd_coef") {
		$listref = [reverse split /\s+/, $2];
		if(@$listref < 1) {
		    die "$fn:$ln: sd_coef must specify at least one number\n";
		}
		foreach(@$listref) {
		    if(! _is_num($_)) {
			die "$fn:$ln: sd_coef must be a list of numbers\n";
		    }
		}
		$this->{"sd_coef"} = $listref;
	    } elsif($1 eq "p_units") {
		$this->{"p_units"} = $2;
	    } elsif($1 eq "r_units") {
		$this->{"r_units"} = $2;
	    } elsif($1 eq "u_range") {
		$listref = [split /\s+/, $2];
		if(@$listref != 2 || ! _is_num($listref->[0]) ||
		   ! _is_num($listref->[1])) {
		    die "$fn:$ln: u_range must be a list of two numbers\n";
		}
		($this->{"u_min"}, $this->{"u_max"}) = @$listref;
	    } elsif($1 eq "sd_range") {
		$listref = [split /\s+/, $2];
		if(@$listref != 2 || ! _is_num($listref->[0]) ||
		   ! _is_num($listref->[1])) {
		    die "$fn:$ln: sd_range must be a list of two numbers\n";
		}
		($this->{"sd_min"}, $this->{"sd_max"}) = @$listref;
	    } else {
		die "$fn:$ln: unknown variable name ($1)\n";
	    }
	} else {
	    die "$fn:$ln: unable to parse name/value pair\n";
	}
    }

    if(! $ispolynomial) {
	die "$fn: This does not seem to be a Model::Polynomial " .
	    "file.  There is no '[Polynomial]' declaration.\n";
    }

    return $this;
}

sub sd {
    my($this, $p) = @_;
    
    if(!$this->{"sd_coef"} || $p < $this->{"sd_min"} || $p > $this->{"sd_max"}){
	return 0;
    }

    my $ret = _eval_polynomial($this->{"sd_coef"}, $p);

    if($ret) {
	return $ret;
    } else {
	return "0 but true";
    }
}

sub predict {
    my($this, $p) = @_;

    if($p < $this->{"u_min"} || $p > $this->{"u_max"}) {
	return 0;
    }

    my $ret = _eval_polynomial($this->{"u_coef"}, $p);

    if($ret) {
	return $ret;
    } else {
	return "0 but true";
    }
}

sub percentile {
    my($this, $p, $r) = @_;

    if($p < $this->{"u_min"} || $p  > $this->{"u_max"} ||
       !$this->{"sd_coef"} ||
       $p < $this->{"sd_min"} || $p > $this->{"sd_max"}) {
	return 0;
    }

    my $ret = (1 - Statistics::Distributions::uprob(
		($r - _eval_polynomial($this->{"u_coef"}, $p))
		    / _eval_polynomial($this->{"sd_coef"}, $p))) * 100;

    if($ret) {
	return $ret;
    } else {
	return "0 but true";
    }
}

sub p_units {
    my($this) = @_;

    return $this->{"p_units"};
}

sub r_units {
    my($this) = @_;
    return $this->{"r_units"};
}

sub _eval_polynomial {
    my($coef, $p) = @_;

    my($i, $ret) = (0, 0);

    foreach(@$coef) {
	$ret += $_ * $p ** $i++;
    }

    return $ret;
}

sub _is_num {
    my($string) = @_;

    if($string =~ m/^[-+]?(?:\d+|\d*\.\d+|\d+\.\d*)(?:[eE][+-]?\d+)?$/) {
	return 1;
    } else {
	return 0;
    }
}

1;
